@@ -32,6 +32,7 @@
 #include <QDateTime>
 #include <QBitArray>
 #include <QRegExp>
+#include <QCoreApplication>
 
 #include <stdlib.h>
 #ifdef HAVE_PTY_H
@@ -51,10 +52,6 @@
 #include <netdb.h>
 #include <sys/types.h>
 
-#ifdef HAVE_STROPTS
-#include <stropts.h>
-#endif
-
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
@@ -133,6 +130,7 @@ extern "C" {
 
 int KDE_EXPORT kdemain( int argc, char **argv )
 {
+    QCoreApplication app(argc, argv);
     KComponentData componentData("fish", "kio_fish");
 
     myDebug( << "*** Starting fish " << endl);
@@ -537,7 +535,7 @@ bool fishProtocol::connectionStart() {
                 outBufLen = 0;
             }
         }
-        if (FD_ISSET(childFd,&rfds)) {
+        else if (FD_ISSET(childFd,&rfds)) {
             rc = ::read(childFd,buf+offset,32768-offset);
             if (rc > 0) {
                 int noff = establishConnection(buf,rc+offset);
@@ -567,8 +565,7 @@ void fishProtocol::writeChild(const QByteArray &buf, KIO::fileoffset_t len) {
     if (outBufPos >= 0 && outBuf.size()) {
 #endif
 #if 0
-        QString debug;
-        debug.setLatin1(outBuf,outBufLen);
+        QString debug = QString::fromLatin1(outBuf,outBufLen);
         if (len > 0) myDebug( << "write request while old one is pending, throwing away input (" << outBufLen << "," << outBufPos << "," << debug.left(10) << "...)" << endl);
 #endif
         return;
@@ -1414,7 +1411,11 @@ void fishProtocol::error(int type, const QString &detail) {
     isRunning = false;
 }
 /** executes a chain of commands */
-void fishProtocol::run() {
+void fishProtocol::run() 
+/* This function writes to childFd fish commands (like #STOR 0 /tmp/test ...) that are stored in outBuf
+and reads from childFd the remote host's response. ChildFd is the fd to a process that communicates
+with .fishsrv.pl typically running on another computer. */
+{
     if (!isRunning) {
         int rc;
         isRunning = true;
@@ -1447,8 +1448,7 @@ void fishProtocol::run() {
             if (outBufPos >= 0) {
 #endif
 #if 0
-                QString debug;
-                debug.setLatin1(outBuf+outBufPos,outBufLen-outBufPos);
+                QString debug = QString::fromLatin1(outBuf+outBufPos,outBufLen-outBufPos);
                 myDebug( << "now writing " << (outBufLen-outBufPos) << " " << debug.left(40) << "..." << endl);
 #endif
 #ifndef Q_WS_WIN
@@ -1479,10 +1479,10 @@ void fishProtocol::run() {
                 }
             }
 #ifndef Q_WS_WIN
-            if (FD_ISSET(childFd,&rfds)) {
+            else if (FD_ISSET(childFd,&rfds)) {
                 rc = ::read(childFd,buf+offset,32768-offset);
 #else
-            if (childPid->waitForReadyRead(1000)) {
+            else if (childPid->waitForReadyRead(1000)) {
                 rc = childPid->read(buf+offset,32768-offset);
 #endif
                 //myDebug( << "read " << rc << " bytes" << endl);
