@@ -17,16 +17,18 @@
    Boston, MA 02110-1301, USA.
 */
 
+#define QT_NO_CAST_FROM_ASCII
+
 #include "kio_trash.h"
 #include <kio/job.h>
 
-#include <kapplication.h>
 #include <kdebug.h>
 #include <klocale.h>
 #include <kde_file.h>
-#include <kcmdlineargs.h>
+#include <kcomponentdata.h>
 #include <kmimetype.h>
 
+#include <QCoreApplication>
 #include <QDataStream>
 #include <QTextStream>
 #include <QFile>
@@ -42,23 +44,12 @@
 extern "C" {
     int KDE_EXPORT kdemain( int argc, char **argv )
     {
-        //KComponentData componentData("kio_trash");
-        // KApplication is necessary to use kio_file
-        putenv(strdup("SESSION_MANAGER="));
-        //KApplication::disableAutoDcopRegistration();
-        KCmdLineArgs::init(argc, argv, "kio_trash", 0, KLocalizedString(), 0);
+        // necessary to use other kio slaves
+        KComponentData componentData("kio_trash" );
+        QCoreApplication app(argc, argv);
 
-        KCmdLineOptions options;
-        options.add("+protocol", ki18n( "Protocol name" ));
-        options.add("+pool", ki18n( "Socket name" ));
-        options.add("+app", ki18n( "Socket name" ));
-        KCmdLineArgs::addCmdLineOptions( options );
-        KApplication app( false );
-
-        KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
-        TrashProtocol slave( QFile::encodeName( args->arg(0) ),
-                             QFile::encodeName( args->arg(1) ),
-                             QFile::encodeName( args->arg(2) ) );
+        // start the slave
+        TrashProtocol slave( argv[1], argv[2], argv[3] );
         slave.dispatchLoop();
         return 0;
     }
@@ -132,7 +123,7 @@ void TrashProtocol::rename( const KUrl &oldURL, const KUrl &newURL, KIO::JobFlag
 
     kDebug()<<"TrashProtocol::rename(): old="<<oldURL<<" new="<<newURL<<" overwrite=" << (flags & KIO::Overwrite);
 
-    if ( oldURL.protocol() == "trash" && newURL.protocol() == "trash" ) {
+    if (oldURL.protocol() == QLatin1String("trash") && newURL.protocol() == QLatin1String("trash")) {
         error( KIO::ERR_CANNOT_RENAME, oldURL.prettyUrl() );
         return;
     }
@@ -146,7 +137,7 @@ void TrashProtocol::copy( const KUrl &src, const KUrl &dest, int /*permissions*/
 
     kDebug()<<"TrashProtocol::copy(): " << src << " " << dest;
 
-    if ( src.protocol() == "trash" && dest.protocol() == "trash" ) {
+    if (src.protocol() == QLatin1String("trash") && dest.protocol() == QLatin1String("trash")) {
         error( KIO::ERR_UNSUPPORTED_ACTION, i18n( "This file is already in the trash bin." ) );
         return;
     }
@@ -156,7 +147,7 @@ void TrashProtocol::copy( const KUrl &src, const KUrl &dest, int /*permissions*/
 
 void TrashProtocol::copyOrMove( const KUrl &src, const KUrl &dest, bool overwrite, CopyOrMove action )
 {
-    if ( src.protocol() == "trash" && dest.isLocalFile() ) {
+    if (src.protocol() == QLatin1String("trash") && dest.isLocalFile()) {
         // Extracting (e.g. via dnd). Ignore original location stored in info file.
         int trashId;
         QString fileId, relativePath;
@@ -191,7 +182,7 @@ void TrashProtocol::copyOrMove( const KUrl &src, const KUrl &dest, bool overwrit
             finished();
         }
         return;
-    } else if ( src.isLocalFile() && dest.protocol() == "trash" ) {
+    } else if (src.isLocalFile() && dest.protocol() == QLatin1String("trash")) {
         QString dir = dest.directory();
         //kDebug() << "trashing a file to " << dir;
         // Trashing a file
@@ -223,7 +214,7 @@ void TrashProtocol::copyOrMove( const KUrl &src, const KUrl &dest, bool overwrit
                 } else {
                     // Inform caller of the final URL. Used by konq_undo.
                     const KUrl url = impl.makeURL( trashId, fileId, QString() );
-                    setMetaData( "trashURL-" + srcPath, url.url() );
+                    setMetaData(QString::fromLatin1("trashURL-") + srcPath, url.url());
                     finished();
                 }
             }
@@ -234,8 +225,9 @@ void TrashProtocol::copyOrMove( const KUrl &src, const KUrl &dest, bool overwrit
             error( KIO::ERR_ACCESS_DENIED, dest.prettyUrl() );
             return;
         }
-    } else
-        error( KIO::ERR_UNSUPPORTED_ACTION, "should never happen" );
+    } else {
+        error(KIO::ERR_UNSUPPORTED_ACTION, i18n("Internal error in copyOrMove, should never happen"));
+    }
 }
 
 void TrashProtocol::createTopLevelDirEntry(KIO::UDSEntry& entry)
@@ -253,7 +245,7 @@ void TrashProtocol::stat(const KUrl& url)
 {
     INIT_IMPL;
     const QString path = url.path();
-    if( path.isEmpty() || path == "/" ) {
+    if (path.isEmpty() || path == QLatin1String("/")) {
         // The root is "virtual" - it's not a single physical directory
         KIO::UDSEntry entry;
         createTopLevelDirEntry( entry );
@@ -275,13 +267,15 @@ void TrashProtocol::stat(const KUrl& url)
             return;
         }
 
+        kDebug() << "parsed" << url << "got" << trashId << fileId << relativePath;
+
         const QString filePath = impl.physicalPath( trashId, fileId, relativePath );
         if ( filePath.isEmpty() ) {
             error( impl.lastErrorCode(), impl.lastErrorMessage() );
             return;
         }
 
-        QString fileName = filePath.section('/', -1, -1, QString::SectionSkipEmpty);
+        QString fileName = filePath.section(QLatin1Char('/'), -1, -1, QString::SectionSkipEmpty);
 
         KUrl fileURL;
         if ( url.path().length() > 1 ) {
@@ -305,6 +299,7 @@ void TrashProtocol::stat(const KUrl& url)
 
 void TrashProtocol::del( const KUrl &url, bool /*isfile*/ )
 {
+    INIT_IMPL;
     int trashId;
     QString fileId, relativePath;
 
@@ -333,7 +328,7 @@ void TrashProtocol::listDir(const KUrl& url)
 {
     INIT_IMPL;
     kDebug() << "listdir: " << url;
-    if ( url.path().length() <= 1 ) {
+    if ( url.path(KUrl::AddTrailingSlash) == QLatin1String("/") ) {
         listRoot();
         return;
     }
@@ -356,7 +351,7 @@ void TrashProtocol::listDir(const KUrl& url)
         return;
     }
     if ( !relativePath.isEmpty() ) {
-        info.physicalPath += '/';
+        info.physicalPath += QLatin1Char('/');
         info.physicalPath += relativePath;
     }
 
@@ -369,14 +364,14 @@ void TrashProtocol::listDir(const KUrl& url)
           entryIt != entryEnd ; ++entryIt )
     {
         const QString fileName = *entryIt;
-        if ( fileName == ".." )
+        if (fileName == QLatin1String(".."))
             continue;
-        const QString filePath = info.physicalPath + '/' + fileName;
+        const QString filePath = info.physicalPath + QLatin1Char('/') + fileName;
         // shouldn't be necessary
         //const QString url = TrashImpl::makeURL( trashId, fileId, relativePath + '/' + fileName );
         entry.clear();
         TrashedFileInfo infoForItem( info );
-        infoForItem.origPath += '/';
+        infoForItem.origPath += QLatin1Char('/');
         infoForItem.origPath += fileName;
         if ( ok && createUDSEntry( filePath, fileName, fileName, entry, infoForItem ) ) {
             listEntry( entry, false );
