@@ -27,24 +27,30 @@
 
 #include <QtGui/QListView>
 #include <QtGui/QPushButton>
+#include <QtGui/QProgressBar>
 
 class QModelIndex;
 class QString;
 
 class ProgressListDelegate::Private
-    : public QObject
 {
-    Q_OBJECT
-
 public:
-    Private(QObject *parent, QListView *listView)
-        : QObject(parent)
-        , listView(listView)
+    Private(QListView *listView)
+        : listView(listView),
+          progressBar(new QProgressBar(0)),
+          separatorPixels(0),
+          leftMargin(0),
+          rightMargin(0),
+          minimumItemHeight(0),
+          minimumContentWidth(0),
+          editorHeight(0),
+          iconWidth(0)
     {
     }
 
     ~Private()
     {
+        delete progressBar;
     }
 
     QString getApplicationInternalName(const QModelIndex &index) const;
@@ -66,19 +72,18 @@ public:
     int getPercent(const QModelIndex &index) const;
     QString getMessage(const QModelIndex &index) const;
     QString getProgressMessage(const QModelIndex &index) const;
-    QStyleOptionProgressBarV2 *getProgressBar(const QModelIndex &index) const;
     int getCurrentLeftMargin(int fontHeight) const;
 
 public:
     int separatorPixels;
     int leftMargin;
     int rightMargin;
-    int progressBarHeight;
     int minimumItemHeight;
     int minimumContentWidth;
     int editorHeight;
     int iconWidth;
     QListView *listView;
+    QProgressBar *progressBar;
 };
 
 #endif // PROGRESSLISTDELEGATE_P_H
