@@ -1,7 +1,4 @@
 /*
-   $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
-
-   This file is part of the Strigi project.
    Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
 
    This library is free software; you can redistribute it and/or
@@ -26,6 +23,7 @@
 #include <strigi/queryparser.h>
 #include <strigi/fieldtypes.h>
 #include "util.h"
+#include "nie.h"
 
 #include <Soprano/Soprano>
 #include <Soprano/Index/IndexFilterModel>
@@ -43,9 +41,11 @@
 #include <QtCore/QDebug>
 #include <QtCore/QString>
 #include <QtCore/QLatin1String>
+#include <QtCore/QFile>
 
 
 using namespace Soprano;
+using namespace std;
 
 
 static lucene::index::Term* createWildCardTerm( const TString& name,
@@ -61,6 +61,17 @@ static lucene::search::Query* createSingleFieldQuery( const string& field,
                                                       const Strigi::Query& query );
 static lucene::search::Query* createMultiFieldQuery( const Strigi::Query& query );
 
+#if 0
+static QString luceneQueryEscape( const QString& s )
+{
+    /* Chars to escape: + - && || ! ( ) { } [ ] ^ " ~  : \ */
+
+    static QRegExp rx( "([\\-" + QRegExp::escape( "+&|!(){}[]^\"~:\\" ) + "])" );
+    QString es( s );
+    es.replace( rx, "\\\\1" );
+    return es;
+}
+#endif
 
 static lucene::index::Term* createWildCardTerm( const TString& name,
                                                 const string& value )
@@ -212,7 +223,7 @@ public:
                     doc.mimetype = value;
                 }
                 else if (fieldName == FieldRegister::mtimeFieldName) {
-                    // FIXME: Sadly in Xesam sourceModified is not typed as DateTime but defaults to an int :( We try to be compatible
+                    // Sadly in Xesam sourceModified is not typed as DateTime but defaults to an int :( We try to be compatible
                     if ( s.object().literal().isDateTime() ) {
                         doc.mtime = s.object().literal().toDateTime().toTime_t();
                     }
@@ -236,7 +247,6 @@ public:
         return true;
     }
 
-//    ::Soprano::Index::IndexFilterModel* repository;
     ::Soprano::Model* repository;
 };
 
@@ -263,9 +273,8 @@ int32_t Strigi::Soprano::IndexReader::countHits( const Query& query )
 
     lucene::search::Query* q = createQuery( query );
     ::Soprano::QueryResultIterator hits = d->repository->executeQuery( TString( q->toString(), true ),
-                                                                       ::Soprano::Query::QUERY_LANGUAGE_USER,
+                                                                       ::Soprano::Query::QueryLanguageUser,
                                                                        QLatin1String( "lucene" ) );
-//    Iterator< ::Soprano::Index::QueryHit> hits = d->repository->index()->search( q );
     int s = 0;
     while ( hits.next() ) {
         qDebug() << "Query hit:" << hits.binding( 0 );
@@ -285,9 +294,8 @@ void Strigi::Soprano::IndexReader::getHits( const Strigi::Query& query,
     qDebug() << "IndexReader::getHits in thread" << QThread::currentThread();
     lucene::search::Query* bq = createQuery( query );
     ::Soprano::QueryResultIterator hits = d->repository->executeQuery( TString( bq->toString(), true ),
-                                                                       ::Soprano::Query::QUERY_LANGUAGE_USER,
+                                                                       ::Soprano::Query::QueryLanguageUser,
                                                                        QLatin1String( "lucene" ) );
-//    Iterator< ::Soprano::Index::QueryHit> hits = d->repository->index()->search( bq );
 
     int i = -1;
     while ( hits.next() ) {
@@ -299,7 +307,6 @@ void Strigi::Soprano::IndexReader::getHits( const Strigi::Query& query,
             break;
         }
 
-//        ::Soprano::Index::QueryHit hit = *hits;
         std::vector<Strigi::Variant> resultRow;
         std::vector<std::string>::const_iterator fieldIt = fields.begin();
         std::vector<Strigi::Variant::Type>::const_iterator typesIt = types.begin();
@@ -336,9 +343,8 @@ std::vector<Strigi::IndexedDocument> Strigi::Soprano::IndexReader::query( const
     vector<IndexedDocument> results;
     lucene::search::Query* bq = createQuery( query );
     ::Soprano::QueryResultIterator hits = d->repository->executeQuery( TString( bq->toString(), true ),
-                                                                       ::Soprano::Query::QUERY_LANGUAGE_USER,
-                                                                       "lucene" );
-//    Iterator< ::Soprano::Index::QueryHit> hits = d->repository->index()->search( bq );
+                                                                       ::Soprano::Query::QueryLanguageUser,
+                                                                       QLatin1String( "lucene" ) );
 
     int i = -1;
     while ( hits.next() ) {
@@ -351,7 +357,6 @@ std::vector<Strigi::IndexedDocument> Strigi::Soprano::IndexReader::query( const
         }
 
         IndexedDocument result;
-//        ::Soprano::Index::QueryHit hit = *hits;
         result.score = hits.binding( 1 ).literal().toDouble();
         if ( d->createDocument( hits.binding( 0 ), result ) ) {
             results.push_back( result );
@@ -369,29 +374,47 @@ std::vector<Strigi::IndexedDocument> Strigi::Soprano::IndexReader::query( const
 void Strigi::Soprano::IndexReader::getChildren( const std::string& parent,
                                                 std::map<std::string, time_t>& children )
 {
-    qDebug() << "IndexReader::getChildren in thread" << QThread::currentThread();
-    QString query = QString( "select distinct ?path ?mtime where { ?r <%1> \"%2\"^^<%3> . ?r <%4> ?mtime . ?r <%5> ?path . }")
-                    .arg( Util::fieldUri( FieldRegister::parentLocationFieldName ).toString() )
-                    .arg( QString::fromUtf8( parent.c_str() ) )
-                    .arg( Vocabulary::XMLSchema::string().toString() )
-                    .arg( Util::fieldUri( FieldRegister::mtimeFieldName ).toString() )
-                    .arg( Util::fieldUri( FieldRegister::pathFieldName ).toString() );
+    //
+    // We are compatible with old Xesam data, thus the weird query
+    //
+    QString query = QString( "select distinct ?path ?mtime where { "
+                             "{ ?r <http://strigi.sf.net/ontologies/0.9#parentUrl> %1 . } "
+                             "UNION "
+                             "{ ?r <http://strigi.sf.net/ontologies/0.9#parentUrl> %2 . } "
+                             "UNION "
+                             "{ ?r %3 %2 . } . "
+                             "{ ?r %4 ?mtime . } UNION { ?r %5 ?mtime . } "
+                             "{ ?r %6 ?path . } UNION { ?r %7 ?path . } "
+                             "}")
+                    .arg( Node::literalToN3( QString::fromUtf8( parent.c_str() ) ),
+                          Node::resourceToN3( QUrl::fromLocalFile( QFile::decodeName( parent.c_str() ) ) ),
+                          Node::resourceToN3( Nepomuk::Vocabulary::NIE::isPartOf() ),
+                          Node::resourceToN3( Vocabulary::Xesam::sourceModified() ),
+                          Node::resourceToN3( Nepomuk::Vocabulary::NIE::lastModified() ),
+                          Node::resourceToN3( Vocabulary::Xesam::url() ),
+                          Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ) );
 
-    qDebug() << "running getChildren query:" << query;
+//    qDebug() << "running getChildren query:" << query;
 
-    QueryResultIterator result = d->repository->executeQuery( query, ::Soprano::Query::QUERY_LANGUAGE_SPARQL );
+    QueryResultIterator result = d->repository->executeQuery( query, ::Soprano::Query::QueryLanguageSparql );
 
     while ( result.next() ) {
         Node pathNode = result.binding( "path" );
         Node mTimeNode = result.binding( "mtime" );
-        qDebug() << "file in index: " << pathNode.toString() << "mtime:" << mTimeNode.literal().toDateTime() << "(" << mTimeNode.literal().toDateTime().toTime_t() << ")";
 
-        // FIXME: Sadly in Xesam sourceModified is not typed as DateTime but defaults to an int :( We try to be compatible
+        // be backwards compatible in case there are paths left encoded as literals
+        std::string path;
+        if ( pathNode.isLiteral() )
+            path = pathNode.toString().toUtf8().data();
+        else
+            path = QFile::encodeName( pathNode.uri().toLocalFile() ).data();
+
+        // Sadly in Xesam sourceModified is not typed as DateTime but defaults to an int :( We try to be compatible
         if ( mTimeNode.literal().isDateTime() ) {
-            children[std::string( pathNode.toString().toUtf8().data() )] = mTimeNode.literal().toDateTime().toTime_t();
+            children[path] = mTimeNode.literal().toDateTime().toTime_t();
         }
         else {
-            children[std::string( pathNode.toString().toUtf8().data() )] = mTimeNode.literal().toUnsignedInt();
+            children[path] = mTimeNode.literal().toUnsignedInt();
         }
     }
 }
@@ -422,22 +445,27 @@ int64_t Strigi::Soprano::IndexReader::indexSize()
 
 time_t Strigi::Soprano::IndexReader::mTime( const std::string& uri )
 {
-    qDebug() << "IndexReader::mTime in thread" << QThread::currentThread();
-    QString query = QString( "select ?mtime where { ?r <%2> \"%3\"^^<%4> . ?r <%1> ?mtime . }" )
-                    .arg( Util::fieldUri( FieldRegister::mtimeFieldName ).toString() )
-                    .arg( Util::fieldUri( FieldRegister::pathFieldName ).toString() )
-                    .arg( QString::fromUtf8( uri.c_str() ) )
-                    .arg( Vocabulary::XMLSchema::string().toString() );
+    //
+    // We are compatible with old Xesam data, thus the weird query
+    //
+    QString query = QString( "select ?mtime where { "
+                             "{ ?r %1 %2 . } UNION { ?r %3 %2 . } "
+                             "{ ?r %4 ?mtime . } UNION { ?r %5 ?mtime . } }" )
+                    .arg( Node::resourceToN3( Vocabulary::Xesam::url() ),
+                          Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ),
+                          Node::literalToN3( QString::fromUtf8( uri.c_str() ) ),
+                          Node::resourceToN3( Vocabulary::Xesam::sourceModified() ),
+                          Node::resourceToN3( Nepomuk::Vocabulary::NIE::lastModified() ) );
 
-    qDebug() << "mTime( " << uri.c_str() << ") query:" << query;
+//    qDebug() << "mTime( " << uri.c_str() << ") query:" << query;
 
-    QueryResultIterator it = d->repository->executeQuery( query, ::Soprano::Query::QUERY_LANGUAGE_SPARQL );
+    QueryResultIterator it = d->repository->executeQuery( query, ::Soprano::Query::QueryLanguageSparql );
 
     time_t mtime = 0;
     if ( it.next() ) {
         ::Soprano::LiteralValue val = it.binding( "mtime" ).literal();
 
-        // FIXME: Sadly in Xesam sourceModified is not typed as DateTime but defaults to an int :( We try to be compatible
+        // Sadly in Xesam sourceModified is not typed as DateTime but defaults to an int :( We try to be compatible
         if ( val.isDateTime() ) {
             mtime = val.toDateTime().toTime_t();
         }
@@ -456,7 +484,7 @@ std::vector<std::string> Strigi::Soprano::IndexReader::fieldNames()
     // Our list of field names (the predicates) is probably awefully long.
 
     std::vector<std::string> fields;
-    QueryResultIterator it = d->repository->executeQuery( "select distinct ?p where { ?r ?p ?o . }", ::Soprano::Query::QUERY_LANGUAGE_SPARQL );
+    QueryResultIterator it = d->repository->executeQuery( "select distinct ?p where { ?r ?p ?o . }", ::Soprano::Query::QueryLanguageSparql );
     while ( it.next() ) {
         fields.push_back( Util::fieldName( it.binding("p").uri() ) );
     }
@@ -468,6 +496,10 @@ std::vector<std::pair<std::string,uint32_t> > Strigi::Soprano::IndexReader::hist
                                                                                        const std::string& fieldname,
                                                                                        const std::string& labeltype )
 {
+    Q_UNUSED(query);
+    Q_UNUSED(fieldname);
+    Q_UNUSED(labeltype);
+
     // FIXME: what is meant by fieldname and labeltype?
     qDebug() << "IndexReader::histogram in thread" << QThread::currentThread();
     // IMPLEMENTME? Seems not like a very important method though.
@@ -478,6 +510,9 @@ std::vector<std::pair<std::string,uint32_t> > Strigi::Soprano::IndexReader::hist
 int32_t Strigi::Soprano::IndexReader::countKeywords( const std::string& keywordprefix,
                                                      const std::vector<std::string>& fieldnames)
 {
+    Q_UNUSED(keywordprefix);
+    Q_UNUSED(fieldnames);
+
     qDebug() << "IndexReader::countKeywords in thread" << QThread::currentThread();
     // the clucene indexer also returns 2. I suspect this means: "not implemented" ;)
     return 2;
@@ -488,6 +523,11 @@ std::vector<std::string> Strigi::Soprano::IndexReader::keywords( const std::stri
                                                                  const std::vector<std::string>& fieldnames,
                                                                  uint32_t max, uint32_t offset )
 {
+    Q_UNUSED(keywordmatch);
+    Q_UNUSED(fieldnames);
+    Q_UNUSED(max);
+    Q_UNUSED(offset);
+
     qDebug() << "IndexReader::keywords in thread" << QThread::currentThread();
     // IMPLEMENTME? Seems like a rarely used method...
     return std::vector<std::string>();
