@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2007-2008 Sebastian Trueg <trueg@kde.org>
+   Copyright (C) 2007-2009 Sebastian Trueg <trueg@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -19,13 +19,15 @@
 
 #include "sopranoindexwriter.h"
 #include "util.h"
+#include "nfo.h"
+#include "nie.h"
 
 #include <Soprano/Soprano>
-#include <Soprano/Index/IndexFilterModel>
-#include <Soprano/Index/CLuceneIndex>
 #include <Soprano/Vocabulary/RDF>
 #include <Soprano/Vocabulary/Xesam>
 #include <Soprano/LiteralValue>
+#include <Soprano/Node>
+#include <Soprano/QueryResultIterator>
 
 #include <QtCore/QList>
 #include <QtCore/QHash>
@@ -36,6 +38,10 @@
 #include <QtCore/QDebug>
 #include <QtCore/QThread>
 #include <QtCore/QDateTime>
+#include <QtCore/QByteArray>
+#include <QtCore/QUuid>
+
+#include <KUrl>
 
 #include <sys/stat.h>
 #include <stdlib.h>
@@ -46,10 +52,14 @@
 #include <sstream>
 #include <algorithm>
 
+#include <Nepomuk/Types/Property>
+#include <Nepomuk/Types/Class>
+#include <Nepomuk/Types/Literal>
 
 // IMPORTANT: strings in Strigi are apparently UTF8! Except for file names. Those are in local encoding.
 
 using namespace Soprano;
+using namespace std;
 
 
 uint qHash( const std::string& s )
@@ -76,28 +86,35 @@ namespace {
         // such as tar:/ or zip:/
         // Here we try to use KDE-compatible URIs for these indexed files the best we can
         // everything else defaults to file:/
+        QUrl uri;
         QString path = QFile::decodeName( idx->path().c_str() );
-        QUrl url = QUrl::fromLocalFile( QFileInfo( path ).absoluteFilePath() );
+        if ( KUrl::isRelativeUrl( path ) )
+            uri = QUrl::fromLocalFile( QFileInfo( path ).absoluteFilePath() );
+        else
+            uri = KUrl( path ); // try to support http and other URLs
+
         if ( idx->depth() > 0 ) {
             QString archivePath = findArchivePath( path );
             if ( QFile::exists( archivePath ) ) {
                 if ( archivePath.endsWith( QLatin1String( ".tar" ) ) ||
                      archivePath.endsWith( QLatin1String( ".tar.gz" ) ) ||
-                     archivePath.endsWith( QLatin1String( ".tar.bz2" ) ) ) {
-                    url.setScheme( "tar" );
+                     archivePath.endsWith( QLatin1String( ".tar.bz2" ) ) ||
+                     archivePath.endsWith( QLatin1String( ".tar.lzma" ) ) ||
+                     archivePath.endsWith( QLatin1String( ".tar.xz" ) ) ) {
+                    uri.setScheme( "tar" );
                 }
                 else if ( archivePath.endsWith( QLatin1String( ".zip" ) ) ) {
-                    url.setScheme( "zip" );
+                    uri.setScheme( "zip" );
                 }
             }
         }
 
         // fallback for all
-        if ( url.scheme().isEmpty() ) {
-            url.setScheme( "file" );
+        if ( uri.scheme().isEmpty() ) {
+            uri.setScheme( "file" );
         }
 
-        return url;
+        return uri;
     }
 
     class FileMetaData
@@ -107,6 +124,9 @@ namespace {
         QUrl fileUri;
         QUrl context;
         std::string content;
+
+        // mapping from blank nodes used in addTriplet to our urns
+        QMap<std::string, QUrl> blankNodeMap;
     };
 
     class RegisteredFieldData
@@ -129,7 +149,8 @@ class Strigi::Soprano::IndexWriter::Private
 {
 public:
     Private()
-        : indexTransactionID( 0 ) {
+        : indexTransactionID( 0 ),
+          currentResult( 0 ) {
         literalTypes[FieldRegister::stringType] = QVariant::String;
         literalTypes[FieldRegister::floatType] = QVariant::Double;
         literalTypes[FieldRegister::integerType] = QVariant::Int;
@@ -139,7 +160,7 @@ public:
 
     QVariant::Type literalType( const Strigi::FieldProperties& strigiType ) {
         // it looks as if the typeUri can contain arbitrary values, URIs or stuff like "string"
-        QHash<std::string, QVariant::Type>::const_iterator it = literalTypes.find( strigiType.typeUri() );
+        QHash<std::string, QVariant::Type>::const_iterator it = literalTypes.constFind( strigiType.typeUri() );
         if ( it == literalTypes.constEnd() ) {
             return LiteralValue::typeFromDataTypeUri( QUrl::fromEncoded( strigiType.typeUri().c_str() ) );
         }
@@ -155,15 +176,57 @@ public:
         if ( type == QVariant::DateTime ) { // dataTime is stored as integer in strigi!
             return LiteralValue( QDateTime::fromTime_t( value.toUInt() ) );
         }
-        else {
+        else if ( type != QVariant::Invalid ) {
             return LiteralValue::fromString( value, type );
         }
+        else {
+            // we default to string
+            return LiteralValue( value );
+        }
+    }
+
+    QUrl createUrn() {
+        QUrl urn;
+        do {
+            urn = QUrl( "urn:nepomuk:local:" + QUuid::createUuid().toString().remove( QRegExp( "[\\{\\}]" ) ) );
+        } while ( repository->executeQuery( QString("ask where { "
+                                                    "{ %1 ?p1 ?o1 . } "
+                                                    "UNION "
+                                                    "{ ?r2 %1 ?o2 . } "
+                                                    "UNION "
+                                                    "{ ?r3 ?p3 %1 . } "
+                                                    "}")
+                                            .arg( ::Soprano::Node::resourceToN3( urn ) ),
+                                            ::Soprano::Query::QueryLanguageSparql ).boolValue() );
+        return urn;
+    }
+
+    QUrl mapNode( FileMetaData* fmd, const std::string& s ) {
+        if ( s[0] == ':' ) {
+            if( fmd->blankNodeMap.contains( s ) ) {
+                return fmd->blankNodeMap[s];
+            }
+            else {
+                QUrl urn = createUrn();
+                fmd->blankNodeMap.insert( s, urn );
+                return urn;
+            }
+        }
+        else {
+            return QUrl::fromEncoded( s.c_str() );
+        }
     }
 
-//    ::Soprano::Index::IndexFilterModel* repository;
     ::Soprano::Model* repository;
     int indexTransactionID;
 
+    //
+    // The Strigi API does not provide context information in addTriplet, i.e. the AnalysisResult.
+    // However, we only use one thread, only one AnalysisResult at the time.
+    // Thus, we can just remember that and use it in addTriplet.
+    //
+    const Strigi::AnalysisResult* currentResult;
+
 private:
     QHash<std::string, QVariant::Type> literalTypes;
 };
@@ -172,10 +235,9 @@ private:
 Strigi::Soprano::IndexWriter::IndexWriter( ::Soprano::Model* model )
     : Strigi::IndexWriter()
 {
-//    qDebug() << "IndexWriter::IndexWriter in thread" << QThread::currentThread();
     d = new Private;
     d->repository = model;
-//    qDebug() << "IndexWriter::IndexWriter done in thread" << QThread::currentThread();
+    Util::storeStrigiMiniOntology( d->repository );
 }
 
 
@@ -193,31 +255,44 @@ void Strigi::Soprano::IndexWriter::commit()
 // delete all indexed data for the files listed in entries
 void Strigi::Soprano::IndexWriter::deleteEntries( const std::vector<std::string>& entries )
 {
-//    qDebug() << "IndexWriter::deleteEntries in thread" << QThread::currentThread();
-
-    QString systemLocationUri = Util::fieldUri( FieldRegister::pathFieldName ).toString();
     for ( unsigned int i = 0; i < entries.size(); ++i ) {
+        //
+        // We are compatible with old Xesam data where the url was encoded as a string instead of a url,
+        // thus the weird query
+        //
         QString path = QString::fromUtf8( entries[i].c_str() );
-        QString query = QString( "select ?g where { ?r <%1> \"%2\"^^<%3> . "
-                                 "?g <http://www.strigi.org/fields#indexGraphFor> ?r . }" )
-                        .arg( systemLocationUri )
-                        .arg( path )
-                        .arg( Vocabulary::XMLSchema::string().toString() );
+        QString query = QString( "select ?g ?mg where { "
+                                 "{ ?r %3 %1 . } "
+                                 "UNION "
+                                 "{ ?r %3 %2 . } "
+                                 "UNION "
+                                 "{ ?r %4 %2 . } . "
+                                 "?g %5 ?r . "
+                                 "OPTIONAL { ?mg %6 ?g . } }" )
+                        .arg( Node::literalToN3( path ),
+                              Node::resourceToN3( QUrl::fromLocalFile( path ) ),
+                              Node::resourceToN3( Vocabulary::Xesam::url() ),
+                              Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ),
+                              Node::resourceToN3( Strigi::Ontology::indexGraphFor() ),
+                              Node::resourceToN3( Vocabulary::NRL::coreGraphMetadataFor() ) );
 
-        qDebug() << "deleteEntries query:" << query;
+//        qDebug() << "deleteEntries query:" << query;
 
-        QueryResultIterator result = d->repository->executeQuery( query, ::Soprano::Query::QUERY_LANGUAGE_SPARQL );
+        QueryResultIterator result = d->repository->executeQuery( query, ::Soprano::Query::QueryLanguageSparql );
         if ( result.next() ) {
             Node indexGraph = result.binding( "g" );
-            result.close();
+            Node metaDataGraph = result.binding( "mg" );
 
-            qDebug() << "Found indexGraph to delete:" << indexGraph;
+            result.close();
 
             // delete the indexed data
             d->repository->removeContext( indexGraph );
 
-            // delete the metadata
-            d->repository->removeAllStatements( Statement( indexGraph, Node(), Node() ) );
+            // delete the metadata (backwards compatible)
+            if ( metaDataGraph.isValid() )
+                d->repository->removeContext( metaDataGraph );
+            else
+                d->repository->removeAllStatements( Statement( indexGraph, Node(), Node() ) );
         }
     }
 }
@@ -225,10 +300,8 @@ void Strigi::Soprano::IndexWriter::deleteEntries( const std::vector<std::string>
 
 void Strigi::Soprano::IndexWriter::deleteAllEntries()
 {
-//    qDebug() << "IndexWriter::deleteAllEntries in thread" << QThread::currentThread();
-
     // query all index graphs (FIXME: would a type derived from nrl:Graph be better than only the predicate?)
-    QString query = QString( "select ?g where { ?g <http://www.strigi.org/fields#indexGraphFor> ?r . }" );
+    QString query = QString( "select ?g where { ?g %1 ?r . }" ).arg( Node::resourceToN3( Strigi::Ontology::indexGraphFor() ) );
 
     qDebug() << "deleteAllEntries query:" << query;
 
@@ -255,24 +328,23 @@ void Strigi::Soprano::IndexWriter::startAnalysis( const AnalysisResult* idx )
         return;
     }
 
-//    qDebug() << "IndexWriter::startAnalysis in thread" << QThread::currentThread();
     FileMetaData* data = new FileMetaData();
     data->fileUri = createResourceUri( idx );
 
     // let's check if we already have data on the file
     StatementIterator it = d->repository->listStatements( Node(),
-                                                          QUrl::fromEncoded( "http://www.strigi.org/fields#indexGraphFor", QUrl::StrictMode ), // FIXME: put the URI somewhere else
+                                                          Strigi::Ontology::indexGraphFor(),
                                                           data->fileUri );
     if ( it.next() ) {
         data->context = it.current().subject().uri();
     }
     else {
-        data->context = Util::uniqueUri( "http://www.strigi.org/contexts/", d->repository );
+        data->context = d->createUrn();
     }
 
-    qDebug() << "Starting analysis for" << data->fileUri << "in thread" << QThread::currentThread();
-
     idx->setWriterData( data );
+
+    d->currentResult = idx;
 }
 
 
@@ -295,41 +367,52 @@ void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
         return;
     }
 
-//    qDebug() << "IndexWriter::addValue in thread" << QThread::currentThread();
     if ( value.length() > 0 ) {
         FileMetaData* md = reinterpret_cast<FileMetaData*>( idx->writerData() );
         RegisteredFieldData* rfd = reinterpret_cast<RegisteredFieldData*>( field->writerData() );
 
+        // the statement we will create, we will determine the object below
+        ::Soprano::Statement statement( md->fileUri, rfd->property, ::Soprano::Node(), md->context );
+
+        //
+        // Strigi uses rdf:type improperly since it stores the value as a string. We have to
+        // make sure it is a resource.
+        //
         if ( rfd->isRdfType ) {
-
-            // Strigi uses rdf:type improperly since it stores the value as a string. We have to
-            // make sure it is a resource. The problem is that this results in the type not being
-            // indexed properly. Thus, it cannot be searched with normal lucene queries.
-            // That is why we need to introduce a stringType property
-
-            d->repository->addStatement( Statement( md->fileUri,
-                                                    ::Soprano::Vocabulary::RDF::type(),
-                                                    QUrl::fromEncoded( value.c_str(), QUrl::StrictMode ), // fromEncoded is faster than the plain constructor and all Xesam URIs work here
-                                                    md->context) );
-            d->repository->addStatement( Statement( md->fileUri,
-                                                    QUrl::fromEncoded( "http://strigi.sourceforge.net/fields#rdf-string-type", QUrl::StrictMode ),
-                                                    LiteralValue( QString::fromUtf8( value.c_str() ) ),
-                                                    md->context) );
+            statement.setPredicate( ::Soprano::Vocabulary::RDF::type() );
+            statement.setObject( QUrl::fromEncoded( value.c_str(), QUrl::StrictMode ) );
         }
 
         else {
-            if ( rfd->dataType != QVariant::Invalid ) {
-                d->repository->addStatement( Statement( md->fileUri,
-                                                        rfd->property,
-                                                        d->createLiteralValue( rfd->dataType, ( unsigned char* )value.c_str(), value.length() ),
-                                                        md->context) );
+            //
+            // we bend the plain strigi properties into something nicer, also because we
+            // do not want paths to be indexed, way too many false positives
+            // in standard desktop searches
+            //
+            if ( field->key() == FieldRegister::pathFieldName ||
+                 field->key() == FieldRegister::parentLocationFieldName ) {
+                // TODO: this is where relative file URLs are to be generated and our new fancy file system
+                // class should provide us with the file system URI
+                statement.setObject( QUrl::fromLocalFile( QFile::decodeName( QByteArray::fromRawData( value.c_str(), value.length() ) ) ) );
             }
             else {
-                qDebug() << "Ignoring field" << rfd->property << "due to unknown type" << field->properties().typeUri().c_str();
+                statement.setObject( d->createLiteralValue( rfd->dataType, ( unsigned char* )value.c_str(), value.length() ) );
+            }
+
+            //
+            // Strigi uses anonymeous nodes prefixed with ':'. However, it is possible that literals
+            // start with a ':'. Thus, we also check the range of the property
+            //
+            if ( value[0] == ':' ) {
+                Nepomuk::Types::Property property( rfd->property );
+                if ( property.range().isValid() ) {
+                    statement.setObject( d->mapNode( md, value ) );
+                }
             }
         }
+
+        d->repository->addStatement( statement );
     }
-//    qDebug() << "IndexWriter::addValue done in thread" << QThread::currentThread();
 }
 
 
@@ -358,7 +441,6 @@ void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
         return;
     }
 
-//    qDebug() << "IndexWriter::addValue in thread" << QThread::currentThread();
     FileMetaData* md = reinterpret_cast<FileMetaData*>( idx->writerData() );
     RegisteredFieldData* rfd = reinterpret_cast<RegisteredFieldData*>( field->writerData() );
 
@@ -371,7 +453,6 @@ void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
                                             rfd->property,
                                             val,
                                             md->context) );
-//    qDebug() << "IndexWriter::addValue done in thread" << QThread::currentThread();
 }
 
 
@@ -383,7 +464,6 @@ void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
         return;
     }
 
-//    qDebug() << "IndexWriter::addValue in thread" << QThread::currentThread();
     FileMetaData* md = reinterpret_cast<FileMetaData*>( idx->writerData() );
     RegisteredFieldData* rfd = reinterpret_cast<RegisteredFieldData*>( field->writerData() );
 
@@ -391,7 +471,6 @@ void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
                                             rfd->property,
                                             LiteralValue( value ),
                                             md->context) );
-//    qDebug() << "IndexWriter::addValue done in thread" << QThread::currentThread();
 }
 
 
@@ -403,7 +482,6 @@ void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
         return;
     }
 
-//    qDebug() << "IndexWriter::addValue in thread" << QThread::currentThread();
     FileMetaData* md = reinterpret_cast<FileMetaData*>( idx->writerData() );
     RegisteredFieldData* rfd = reinterpret_cast<RegisteredFieldData*>( field->writerData() );
 
@@ -411,52 +489,70 @@ void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
                                             rfd->property,
                                             LiteralValue( value ),
                                             md->context) );
-//    qDebug() << "IndexWriter::addValue done in thread" << QThread::currentThread();
 }
 
 
-void Strigi::Soprano::IndexWriter::addTriplet( const std::string& subject,
-                                               const std::string& predicate, const std::string& object )
+void Strigi::Soprano::IndexWriter::addTriplet( const std::string& s,
+                                               const std::string& p,
+                                               const std::string& o )
 {
-    // PROBLEM: which named graph (context) should we use here? Create a new one for each triple? Use one until the
-    // next commit()?
+    //
+    // The Strigi API does not provide context information here, i.e. the AnalysisResult this triple
+    // belongs to. However, we only use one thread, only one AnalysisResult at the time.
+    // Thus, we can just remember that and use it here.
+    //
 
-    // FIXME: create an NRL metadata graph
-    d->repository->addStatement( Statement( Node( QUrl( QString::fromUtf8( subject.c_str() ) ) ),
-                                            Node( QUrl( QString::fromUtf8( predicate.c_str() ) ) ),
-                                            Node( QUrl( QString::fromUtf8( object.c_str() ) ) ),
-                                            Node() ) );
+    FileMetaData* md = static_cast<FileMetaData*>( d->currentResult->writerData() );
+
+    QUrl subject = d->mapNode( md, s );
+    Nepomuk::Types::Property property( d->mapNode( md, p ) );
+    ::Soprano::Node object;
+    if ( property.range().isValid() )
+        object = d->mapNode( md, o );
+    else
+        object = ::Soprano::LiteralValue::fromString( QString::fromUtf8( o.c_str() ), property.literalRangeType().dataTypeUri() );
+
+    d->repository->addStatement( subject, property.uri(), object, md->context );
 }
 
 
 // called after each indexed file
 void Strigi::Soprano::IndexWriter::finishAnalysis( const AnalysisResult* idx )
 {
+    d->currentResult = 0;
+
     if ( idx->depth() > 0 ) {
         return;
     }
 
-//    qDebug() << "IndexWriter::finishAnalysis in thread" << QThread::currentThread();
     FileMetaData* md = static_cast<FileMetaData*>( idx->writerData() );
 
     if ( md->content.length() > 0 ) {
         d->repository->addStatement( Statement( md->fileUri,
-                                                Vocabulary::Xesam::asText(),
+                                                Nepomuk::Vocabulary::NIE::plainTextContent(),
                                                 LiteralValue( QString::fromUtf8( md->content.c_str() ) ),
                                                 md->context ) );
+        if ( d->repository->lastError() )
+            qDebug() << "Failed to add" << md->fileUri << "as text" << QString::fromUtf8( md->content.c_str() );
     }
 
-    // Strigi only indexes files and extractors mostly (if at all) store the xesam:DataObject type (i.e. the contents)
-    // Thus, here we go the easy way and mark each indexed file as a xesam:File.
-    d->repository->addStatement( Statement( md->fileUri,
-                                            Vocabulary::RDF::type(),
-                                            Vocabulary::Xesam::File(),
-                                            md->context ) );
+    // Strigi only indexes files and extractors mostly (if at all) store the nie:DataObject type (i.e. the contents)
+    // Thus, here we go the easy way and mark each indexed file as a nfo:FileDataObject.
+    if ( QFileInfo( QFile::decodeName( idx->path().c_str() ) ).isDir() )
+        d->repository->addStatement( Statement( md->fileUri,
+                                                Vocabulary::RDF::type(),
+                                                Nepomuk::Vocabulary::NFO::Folder(),
+                                                md->context ) );
+    else
+        d->repository->addStatement( Statement( md->fileUri,
+                                                Vocabulary::RDF::type(),
+                                                Nepomuk::Vocabulary::NFO::FileDataObject(),
+                                                md->context ) );
 
 
     // create the provedance data for the data graph
     // TODO: add more data at some point when it becomes of interest
-    QUrl metaDataContext = Util::uniqueUri( "http://www.strigi.org/graphMetaData/", d->repository );
+    QUrl metaDataContext = md->context.toString() + "-metadata";
     d->repository->addStatement( Statement( md->context,
                                             Vocabulary::RDF::type(),
                                             Vocabulary::NRL::InstanceBase(),
@@ -466,19 +562,21 @@ void Strigi::Soprano::IndexWriter::finishAnalysis( const AnalysisResult* idx )
                                             LiteralValue( QDateTime::currentDateTime() ),
                                             metaDataContext ) );
     d->repository->addStatement( Statement( md->context,
-                                            QUrl::fromEncoded( "http://www.strigi.org/fields#indexGraphFor", QUrl::StrictMode ), // FIXME: put the URI somewhere else
+                                            Strigi::Ontology::indexGraphFor(),
                                             md->fileUri,
                                             metaDataContext ) );
     d->repository->addStatement( Statement( metaDataContext,
                                             Vocabulary::RDF::type(),
                                             Vocabulary::NRL::GraphMetadata(),
                                             metaDataContext ) );
+    d->repository->addStatement( metaDataContext,
+                                 Vocabulary::NRL::coreGraphMetadataFor(),
+                                 md->context,
+                                 metaDataContext );
 
     // cleanup
     delete md;
     idx->setWriterData( 0 );
-
-//    qDebug() << "IndexWriter::finishAnalysis done in thread" << QThread::currentThread();
 }
 
 
